"""Contains class of Calls Process."""
from grortir.main.model.core.abstract_process import AbstractProcess


class CallsProcess(AbstractProcess):
    """Implementation of process with the same type of stages - calls."""
